/*

 xtract_c

 Extract the text immediately following a key string from all files
 matching a specification.

 Date		Author		Comments
 ----		------		--------
 25/03/1998	A. D. Ives	First version.

*/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <qdos.h>
#include <unistd.h>
#include <ctype.h>

char _PROG_NAME[]= "xtract" ;
long _stack= 16384 ;

#define PROGRAM_VERSION "1.00"
#define MAX_BUFF 4096

#define TRUE 1
#define FALSE 0

void help() {

  /* Display the options */

  printf("\nXtract v%s (c) Adrian D. Ives\n\n", PROGRAM_VERSION) ;

  printf("Extract the text following the first occurrence of a key string\n") ;
  printf("in the specified (data only) file(s).\n\n") ;

  printf("Use:  xtract FileSpec [-fKeyString] [-sSortOpt] [Switches]\n\n") ;

  printf("      If not specified, the default KeyString is 'subject:'\n\n") ;

  printf("SortOpt Ascend  Descend On\n") ;
  printf("        N       n       Name\n") ;
  printf("        U       u       Type (use)\n") ;
  printf("        S       s       Size\n") ;
  printf("        D       d       Date\n") ;
  printf("        T       t       Time\n\n") ;

  printf("-Number  Find nth occurrence (Number = any integer)\n") ;
  printf("-t       Use Tab as a separator\n") ;
  printf("-m       Match case\n\n") ;

}

char sort[20], spec[64], find[128] ;

int which ,
    UseTabs ,
    MatchCase ;

void xtract() {

  /*

    This is where all the hard work is done!

    Don't know why, but if you try to do this code inside main()
    when the directory has a large number of files we get a
    really major memory trash when qdir_read() is called.

    nb: This might be something to do with stack.

  */

  int devend, bytes, flen, eol, nth, n ;
  DIR_LIST_t *ptr, *dir ;
  qdirect_t this ;
  char fname[64], buff[MAX_BUFF+2], buff2[MAX_BUFF+2], *fn, *found, *bptr ;
  FILE *current ;

  if ( !MatchCase )
    strlwr(find) ;
  flen= strlen(find) ;

  if ( !*spec ) {
    getcwd(spec, 64) ;
    strcat(spec, "*");
  }

  if ( !isdirdev(spec) ) {

    char temp[64] ;

    getcwd(temp, 64) ;
    strcat(temp, spec) ;
    strcpy(spec, temp) ;
  }

  if ( strpos(spec, '*') == -1 && strpos(spec, '?') == -1 && strpos(spec, '[') == -1 )
    strcat(spec, "*") ;

  devend= strpos(spec, '_') ;
  strncpy(fname, spec, ++devend) ;
  fname[devend]= 0 ;
  strupr(fname) ;
  fn= fname + devend ;

  dir= qdir_read(spec, sort, 0) ;
  ptr= dir ;
  while ( ptr ) {
    this= ptr->dl_dir ;
    strcpy(fn, this.d_name) ;
    current= fopen(fname, "r") ;
    if ( current != NULL ) {
      found= NULL ;
      bytes= 1 ;
      nth= which ;
      bptr= buff ;
      while ( (found == NULL) && (bytes) ) {
	if ( bptr != buff )

	  /* Not the first read, so start where we left off */

	  n= strlen(bptr) ;
	else
	  n= 0 ;

	if ( n ) {
	  strcpy(buff2, bptr) ;
	  strcpy(buff, buff2) ;
	  bytes= fread(buff + n, 1, MAX_BUFF - n, current) + n ;
	  buff[bytes]= 0 ;
	}
	else {
	  bytes= fread(buff, 1, MAX_BUFF, current) ;
	  buff[bytes]= 0 ;
	}

	bptr= buff ;
	found= bptr ; /* Just to kick the loop off! */
	while ( found != NULL && *bptr && nth ) {
	  if ( MatchCase )
	    found= strstr(bptr, find) ;
	  else {
	    strcpy(buff2, bptr) ;
	    strlwr(buff2) ;
	    found= strstr(buff2, find) ;
	    if ( found != NULL )
	      found= bptr + (found - buff2) ;
	  }
	  if ( found != NULL ) {
	    found+= flen ;
	    if ( --nth == 0 ) {
	      eol= strpos(found, '\n') ;
	      if ( eol != -1 )
		found[eol]= 0 ;
	      while ( *found && *found == ' ' )  /* Skip leading blanks */
		found++ ;
	      if ( UseTabs )
		printf("%s\t%s\n", fname, found) ;
	      else
		printf("%s %s\n", fname, found) ;
	    }
	    else
	      bptr= found ;
	  } /* if ( found != NULL ) */
	}   /* while ( found != NULL && *bptr && nth ) */
      }
      fclose(current) ;
    }
    else
      printf("* Couldn't Open: %s *\n", fname) ;
    ptr= ptr->dl_next ;
  }
  qdir_delete(dir) ;
}

main(int argc, char **argv) {

  int i ;

  strcpy(find, "subject:") ;
  strcpy(sort, "N") ;
  *spec= 0 ;
  which= 1 ;
  UseTabs= MatchCase= FALSE ;
  for ( i= 1 ; i <= argc - 1 ; i++ ) {
    if ( argv[i][0] == '-' ) {
      if ( isdigit(argv[i][1]) )
	which= atol(argv[i] + 1) ;
      else {
	switch( argv[i][1] ) {
	  case	0  :
	  case '?' :
	    help() ;
	    return 0 ;
	  case 'f' :
	    strcpy(find, argv[i] + 2) ;
	    break ;
	  case 'm' :
	    MatchCase= TRUE ;
	    break ;
	  case 's' :
	    if ( argv[i][2] )
	      strcpy(sort, argv[i] + 2) ;
	    break ;
	  case 't' :
	    UseTabs= TRUE ;
	    break ;
	}
      }
    }
    else
      strcpy(spec, argv[i]) ;
  }
  xtract() ;
  return 0 ;
}
